#version 330 core
#define NR_POINT_LIGHTS 5

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 fragPosLightSpace;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform sampler2D depthMap;
uniform int textureMultiplikatorX =1;
uniform int textureMultiplikatorY =1;

uniform vec3 lightPos;

uniform samplerCube cubeDepthMap1;
uniform samplerCube cubeDepthMap2;
uniform samplerCube cubeDepthMap3;
uniform samplerCube cubeDepthMap4;
uniform vec3 pointLightPos;
uniform float pointLightFarPlane;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
};
uniform PointLight pointLights[NR_POINT_LIGHTS];

vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

vec3 phongPointLight(vec3 n, vec3 v, vec3 texColor, vec3 specTexColor, vec3 lightDir, vec3 pointLcolor, vec3 attenuation) {
	float distance = length(lightDir);
	lightDir = normalize(lightDir);

	float att = 1.0f / (attenuation.x + distance * attenuation.y + distance * distance * attenuation.z);
	vec3 refelctionDir = reflect(-lightDir, n);

	vec3 diffuseColor = pointLcolor * texColor;
	diffuseColor = materialCoefficients.y * diffuseColor * max(0, dot(n, lightDir));
	vec3 specularColor = materialCoefficients.z * pointLcolor * specTexColor * pow(max(0, dot(refelctionDir, v)), specularAlpha);
	vec3 ambientColor = texColor * materialCoefficients.x;

	return diffuseColor*att + specularColor*att; //ambientColor/4 + 
}

float linearizeDepth(float depth) {
	float z = depth*2-1;
	return(2*0.1*200)/(200+0.1-z*(200-0.1));
}

float calculateShadow(vec4 fragPosLightSpace) {
	vec3 normal = normalize(vert.normal_world);
	vec3 lightDir = normalize(lightPos - vert.position_world);
	float bias = max(0.005 * (1.0 - dot(normal, lightDir)), 0.0001);
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5;

    float currentDepth = projCoords.z;
	vec2 texelSize = 2.0/textureSize(depthMap,0);
	float shadow;// = currentDepth - 0.0005 > closestDepth  ? 1.0 : 0.0;

	texelSize = 1.0/textureSize(depthMap,0);

	for(int i = -4; i<4; i++) {
		for(int j = -4; j<4; j++) {
			float closestDepth = texture(depthMap, projCoords.xy + vec2(i,j)*texelSize).r; 
			shadow += currentDepth - bias > closestDepth  ? 1.0 : 0.0;
		}
	}
	return shadow/64;
}

float calculatePointShadow(vec3 fragPos, vec3 pointLightPos, samplerCube cubeDepthMap) {
	vec3 fragToLight = fragPos - pointLightPos;
	float currentDepth = length(fragToLight);
	float bias = 0.7;
	
    //float closestDepth = texture(cubeDepthMap, fragToLight).r;
    //closestDepth *= pointLightFarPlane;
	//float shadow = (currentDepth-bias > closestDepth) ? 1.0 : 0.0;
	//return shadow;
	float shadow = 0;
	float samples = 20;
	float diskRadius = (1.0 + (currentDepth/pointLightFarPlane)) /25.0;
	for(int i = 0; i < samples; ++i)
    {
        float closestDepth = texture(cubeDepthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
        closestDepth *= pointLightFarPlane;
        if(currentDepth - bias > closestDepth)
            shadow += 1.0;
    }
	shadow= shadow/samples;
	return shadow;
}

void main()
{
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec2 newUV = vec2(vert.uv.x*textureMultiplikatorX, vert.uv.y*textureMultiplikatorY);
	vec3 texColor1 = texture(diffuseTexture, newUV).rgb;
	vec3 texColor2 = texture(specularTexture, newUV).rgb;
	vec3 texColor = mix(texColor1, texColor2, 0.1);

	vec3 lightColor;

	vec3 ambientColor = texColor * materialCoefficients.x;

	float shadow1 = calculatePointShadow(vert.position_world, pointLights[1].position, cubeDepthMap1);
	float shadow2 = calculatePointShadow(vert.position_world, pointLights[2].position, cubeDepthMap2);
	float shadow3 = calculatePointShadow(vert.position_world, pointLights[3].position, cubeDepthMap3);
	float shadow4 = calculatePointShadow(vert.position_world, pointLights[4].position, cubeDepthMap4);
	float shadow = shadow1+shadow2+shadow3+shadow4;//calculateShadow(vert.fragPosLightSpace);
	shadow = shadow/4;
	texColor = texColor * (1.0-shadow);
	texColor2 = texColor2 * (1.0-shadow);
	for(int i = 0; i< NR_POINT_LIGHTS; i++) {
		lightColor += phongPointLight(n,v,texColor, texColor2, pointLights[i].position - vert.position_world, pointLights[i].color, pointLights[i].attenuation);
	}
	lightColor = lightColor;
	lightColor += ambientColor;

	vec3 camDistVec = camera_world - vert.position_world;
	float camDist = length(camDistVec);
	float fog = min(1, 1.0f / (1 + camDist*0.007 + camDist*camDist*0.0002));
	texColor = lightColor * fog;


	//texColor = texColor*(1-shadow);

	float depth = 1 - linearizeDepth(gl_FragCoord.z) / 200;
	color = vec4(texColor*depth, 1); //vec4(vec3(depth),1.0);
	//color= vec4(vec3(shadow), 1);
}